/*
 * GKLJ 0.1.3 - notifies you when your LiveJournal friends page is updated
 * Copyright 2001 Austin Jennings <axial@users.sourceforge.net>
 *
 * GKLJ is protected under the terms of the GNU General Public License.
 * See the file 'COPYING' for details.
 */

#define CLIENTVER "GKrellM-GKLJ/0.1.3"

#include <errno.h>
#include <fcntl.h>
#include <gkrellm/gkrellm.h>
#include <netdb.h>
#include <netinet/in.h>
#include <stdarg.h>
#include <stdio.h>
#include <string.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <sys/types.h>
#include <time.h>
#include <unistd.h>

#include "config.h"
#include "gklj.h"
#include "md5.h"

static Panel *panel;
static Decal *led, *text;
static gint style_id;
static gint led_data = D_MISC_LED0;
static gchar text_data[8] = "";
static GtkWidget *username_entry, *password_entry;
static GtkWidget *browser_entry, *notify_entry;
static gchar username[64] = "", password[33] = "";
static gchar browser[512] = "", notify[512] = "";
static GString *result, *gbuf, *lastupdate;
static gboolean fastserver = FALSE;
static enum State state = STATE_LOGIN_WAIT;
static gint interval = 0;
static gboolean paused = FALSE, relogin;

static gchar *info_text[] = {
	"<b>" CLIENTVER "\n",
	"GKLJ monitors your LiveJournal friends page and notifies you when\n"
	"it's been updated.  The GKLJ panel sits under the mailcheck.  When\n"
	"a new message appears on your friends page, the LED will light up\n"
	"and the word 'new' will appear on the panel.  Left-clicking the\n"
	"panel will reset it and load your friends page.  A right-click will\n"
	"reset the update indicator without launching your browser.\n"
	"\n",
	"<b>Command to view friends page\n",
	"This is the command that will be run when you left-click the panel.\n"
	"For example, ",
	"<i>mozilla http://www.livejournal.com/users/test/friends/\n",
	"\n",
	"<b>Notify (sound) program\n",
	"If you specify a notify program, it will be run whenever GKLJ\n"
	"detects a new message on your friends page.\n"
};

static Monitor plugin_mon = {
	"GKLJ",			/* name */
	0,			/* id */
	create_plugin,		/* create_plugin function */
	update_plugin,		/* update_plugin function */
	create_plugin_tab,	/* create_plugin_tab function */
	apply_plugin_config,	/* apply_plugin_config function */
	save_plugin_config,	/* save_plugin_config function */
	load_plugin_config,	/* load_plugin_config function */
	"gklj",			/* config keyword */
	NULL,			/* undefined */
	NULL,			/* undefined */
	NULL,			/* undefined */
	PLUGIN_PLACEMENT,	/* plugin placement */
	NULL,			/* handle */
	NULL			/* path */
};

/* init_plugin: initialize plugin and returns a pointer to plugin_mon */
Monitor *init_plugin()
{
	/* initialize variables */
	result = g_string_new("");
	gbuf = g_string_new("");
	lastupdate = g_string_new("");
	style_id = gkrellm_add_meter_style(&plugin_mon, "gklj");

	return &plugin_mon;
}

/* create_plugin: construct plugin panel */
static void create_plugin(GtkWidget *vbox, gint first_create)
{
	Style *style;
	TextStyle *ts;
	GdkPixmap *pixmap;
	GdkBitmap *mask;

	if (first_create)
		panel = gkrellm_panel_new0();
	else
		gkrellm_destroy_decal_list(panel);

	style = gkrellm_meter_style(style_id);

	/* create a pixmap decal */
	pixmap = gkrellm_decal_misc_pixmap();
	mask = gkrellm_decal_misc_mask();
	led =
	    gkrellm_create_decal_pixmap(panel, pixmap, mask, N_MISC_DECALS,
					NULL, 0, 0);

	/* create a test decal */
	ts = gkrellm_meter_textstyle(style_id);
	text =
	    gkrellm_create_decal_text(panel, "Ay", ts, style, led->w, -1,
				      -1);

	/* center led vertically */
	led->y = text->h / 2 - 2;

	/* configure the panel */
	gkrellm_configure_panel(panel, NULL, style);
	gkrellm_create_panel(vbox, panel,
			     gkrellm_bg_meter_image(style_id));
	gkrellm_monitor_height_adjust(panel->h);

	/* put text in a button (just for looks, we handle clicks elsewhere) */
	gkrellm_put_decal_in_meter_button(panel, text, (void *) NULL,
					  GINT_TO_POINTER(1));

	redraw();

	if (first_create) {
		gtk_signal_connect(GTK_OBJECT(panel->drawing_area),
				   "expose_event", (GtkSignalFunc)
				   panel_expose_event, NULL);
		gtk_signal_connect(GTK_OBJECT(panel->drawing_area),
				   "button_press_event", (GtkSignalFunc)
				   panel_button_press_event, NULL);
	}
}

/* update_plugin: called a few times per second, here's where we actually do
 * polling and such. */
void update_plugin()
{
	extern int errno;
	static struct hostent *he;
	static int s;
	static struct sockaddr_in sa;
	static gchar buf[1024];
	static ssize_t rv;

	if (GK.second_tick)
		interval--;

	switch (state) {
	case STATE_LOGIN_WAIT:
		if (interval > 0)
			break;

		relogin = FALSE;

		if (!strcmp(username, "") || !strcmp(password, "")) {
			/* plugin unconfigured */
			strcpy(text_data, "config");
			redraw();
			interval = MIN_INTERVAL;
			break;
		}

		if ((he = gethostbyname(LIVEJOURNAL_HOST)) == NULL) {
			/* can't resolve hostname */
			strcpy(text_data, "host");
			redraw();
			interval = MIN_INTERVAL;
			break;
		}

		bzero(&sa, sizeof(sa));
		bcopy(he->h_addr, (char *) &sa.sin_addr, he->h_length);
		sa.sin_family = he->h_addrtype;
		sa.sin_port = htons((u_short) LIVEJOURNAL_PORT);

		if (prepare_socket(&s, he)) {
			/* can't prepare socket */
			interval = MIN_INTERVAL;
			break;
		}

		/* we'll use interval to count seconds until connect
		 * times out */
		interval = TIMEOUT;

		state = STATE_LOGIN_CONNECT;
		break;
	case STATE_LOGIN_CONNECT:
		if (interval <= 0) {
			/* timeout */
#ifdef GKLJ_DEBUG
			strcpy(text_data, "timeout");
			redraw();
#endif
			state = STATE_LOGIN_WAIT;
			interval = MIN_INTERVAL;
			close(s);
		} else if (connect(s, (struct sockaddr *) &sa, sizeof
				   (sa)) < 0) {
			/* connect failed */
			if (errno != EINPROGRESS && errno != EALREADY) {
#ifdef GKLJ_DEBUG
				strcpy(text_data, "connect");
				redraw();
				perror("connect");
#endif
				state = STATE_LOGIN_WAIT;
				interval = MIN_INTERVAL;
				close(s);
			}
		} else {
			/* connect succeeded */
			g_string_assign(gbuf, request(8,
						      "mode", "login",
						      "user", username,
						      "hpassword",
						      password,
						      "clientversion",
						      CLIENTVER));
#ifdef GKLJ_DEBUG
			printf("--WRITING--\n%s", gbuf->str);
#endif

			/* we'll use interval to count seconds
			 * until write times out */
			interval = TIMEOUT;

			state = STATE_LOGIN_WRITE;
		}
		break;
	case STATE_LOGIN_WRITE:
		if (interval <= 0) {
			/* timeout */
#ifdef GKLJ_DEBUG
			strcpy(text_data, "timeout");
			redraw();
#endif
			state = STATE_LOGIN_WAIT;
			interval = MIN_INTERVAL;
			close(s);
		} else if ((rv = write(s, gbuf->str, gbuf->len)) < 0) {
			/* failed write */
			if (errno != EAGAIN && errno != EINTR) {
#ifdef GKLJ_DEBUG
				strcpy(text_data, "write");
				redraw();
				perror("write");
#endif
				state = STATE_LOGIN_WAIT;
				interval = MIN_INTERVAL;
				close(s);
			}
		} else if (rv) {
			/* remove what we wrote from gbuf */
			if (rv != gbuf->len)
				g_string_erase(gbuf, 0, rv);
			else {
				/* all done.. initialize variables for
				 * read state */
				g_string_assign(gbuf, "");
				interval = TIMEOUT;
				state = STATE_LOGIN_READ;
			}
		}
		break;
	case STATE_LOGIN_READ:
		if (interval <= 0) {
			/* timeout */
#ifdef GKLJ_DEBUG
			strcpy(text_data, "timeout");
			redraw();
#endif
			state = STATE_LOGIN_WAIT;
			interval = MIN_INTERVAL;
			close(s);
		} else if ((rv = read(s, buf, 1024)) < 0) {
			/* failed read */
			if (errno != EINTR && errno != EAGAIN) {
#ifdef GKLJ_DEBUG
				strcpy(text_data, "read");
				redraw();
				perror("read");
#endif
				state = STATE_LOGIN_WAIT;
				interval = MIN_INTERVAL;
				close(s);
			}
		} else if (rv) {
			/* new data, write it to gbuf */
			buf[rv] = '\0';
			g_string_append(gbuf, buf);
		} else {
			/* all done */
#ifdef GKLJ_DEBUG
			printf("--READING--\n%s", gbuf->str);
#endif
			close(s);
			if (parse_login(gbuf->str)) {
				/* login unsuccessful */
				strcpy(text_data, "login");
				redraw();
				state = STATE_LOGIN_WAIT;
				interval = MIN_INTERVAL;
			} else {
				/* login successful */
				state = STATE_CHECKFRIENDS_WAIT;
				paused = FALSE;
				interval = 0;
				g_string_assign(lastupdate, "");
			}
		}
		break;
	case STATE_CHECKFRIENDS_WAIT:
		if (relogin) {
			state = STATE_LOGIN_WAIT;
			interval = 0;
			break;
		}

		if (interval > 0 || paused)
			break;

		if (prepare_socket(&s, he)) {
			/* can't prepare socket */
			interval = MIN_INTERVAL;
			break;
		}

		/* we'll use interval to count seconds until connect
		 * times out */
		interval = TIMEOUT;

		state = STATE_CHECKFRIENDS_CONNECT;
		break;
	case STATE_CHECKFRIENDS_CONNECT:
		if (interval <= 0) {
			/* timeout */
#ifdef GKLJ_DEBUG
			strcpy(text_data, "timeout");
			redraw();
#endif
			state = STATE_CHECKFRIENDS_WAIT;
			interval = MIN_INTERVAL;
			close(s);
		} else if (connect(s, (struct sockaddr *) &sa, sizeof
				   (sa)) < 0) {
			if (errno != EINPROGRESS && errno != EALREADY) {
				/* connect failed */
#ifdef GKLJ_DEBUG
				strcpy(text_data, "connect");
				redraw();
				perror("connect");
#endif
				state = STATE_CHECKFRIENDS_WAIT;
				interval = MIN_INTERVAL;
				close(s);
			}
		} else {
			/* connect succeeded */
			g_string_assign(gbuf, request(8,
						      "mode",
						      "checkfriends",
						      "user", username,
						      "hpassword",
						      password,
						      "lastupdate",
						      lastupdate->str));
#ifdef GKLJ_DEBUG
			printf("--WRITING--\n%s", gbuf->str);
#endif

			/* use interval to count seconds until write
			 * times out */
			interval = TIMEOUT;

			state = STATE_CHECKFRIENDS_WRITE;
		}
		break;
	case STATE_CHECKFRIENDS_WRITE:
		if (interval <= 0) {
			/* timeout */
#ifdef GKLJ_DEBUG
			strcpy(text_data, "timeout");
			redraw();
#endif
			state = STATE_CHECKFRIENDS_WAIT;
			interval = MIN_INTERVAL;
			close(s);
		} else if ((rv = write(s, gbuf->str, gbuf->len)) < 0) {
			/* failed write */
			if (errno != EAGAIN && errno != EINTR) {
#ifdef GKLJ_DEBUG
				strcpy(text_data, "write");
				redraw();
				perror("write");
#endif
				state = STATE_CHECKFRIENDS_WAIT;
				interval = MIN_INTERVAL;
				close(s);
			}
		} else if (rv) {
			/* remove what we wrote from gbuf */
			if (rv != gbuf->len)
				g_string_erase(gbuf, 0, rv);
			else {
				/* all done.. initialize variables for
				 * read state */
				g_string_assign(gbuf, "");
				interval = TIMEOUT;
				state = STATE_CHECKFRIENDS_READ;
			}
		}
		break;
	case STATE_CHECKFRIENDS_READ:
		if (interval <= 0) {
			/* timeout */
#ifdef GKLJ_DEBUG
			strcpy(text_data, "timeout");
			redraw();
#endif
			state = STATE_CHECKFRIENDS_WAIT;
			interval = MIN_INTERVAL;
			g_string_assign(lastupdate, "");
			close(s);
		} else if ((rv = read(s, buf, 1024)) < 0) {
			/* failed read */
			if (errno != EINTR && errno != EAGAIN) {
#ifdef GKLJ_DEBUG
				strcpy(text_data, "read");
				redraw();
				perror("read");
#endif
				state = STATE_CHECKFRIENDS_WAIT;
				interval = MIN_INTERVAL;
				g_string_assign(lastupdate, "");
				close(s);
			}
		} else if (rv) {
			/* new data, write it to gbuf */
			buf[rv] = '\0';
			g_string_append(gbuf, buf);
		} else {
			/* all done */
#ifdef GKLJ_DEBUG
			printf("--READING--\n%s", gbuf->str);
#endif
			close(s);
			if (parse_checkfriends(gbuf->str)) {
				/* checkfriends unsuccessful */
				strcpy(text_data, "failed");
				redraw();
				state = STATE_CHECKFRIENDS_WAIT;
				interval = MIN_INTERVAL;
				g_string_assign(lastupdate, "");
			}

			state = STATE_CHECKFRIENDS_WAIT;
		}
	}
}

/* create_plugin_tab: construct the configuration tab */
static void create_plugin_tab(GtkWidget *tab_vbox)
{
	GtkWidget *tabs, *vbox, *label, *scrolled, *text;

	/* create tabs */
	tabs = gtk_notebook_new();
	gtk_notebook_set_tab_pos(GTK_NOTEBOOK(tabs), GTK_POS_TOP);
	gtk_box_pack_start(GTK_BOX(tab_vbox), tabs, TRUE, TRUE, 0);

	/* setup tab */
	vbox = gkrellm_create_tab(tabs, "Setup");
	label = gtk_label_new(_("LiveJournal username:"));
	gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);
	username_entry = gtk_entry_new();
	gtk_entry_set_max_length(GTK_ENTRY(username_entry), 63);
	gtk_entry_set_text(GTK_ENTRY(username_entry), username);
	gtk_box_pack_start(GTK_BOX(vbox), username_entry, FALSE, FALSE, 0);
	label = gtk_label_new(_("LiveJournal password:"));
	gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);
	password_entry = gtk_entry_new();
	gtk_entry_set_max_length(GTK_ENTRY(password_entry), 32);
	gtk_entry_set_visibility(GTK_ENTRY(password_entry), FALSE);
	gtk_entry_set_text(GTK_ENTRY(password_entry), password);
	gtk_box_pack_start(GTK_BOX(vbox), password_entry, TRUE, TRUE, 0);
	label = gtk_label_new(_("Command to view friends page:"));
	gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);
	browser_entry = gtk_entry_new();
	gtk_entry_set_max_length(GTK_ENTRY(browser_entry), 511);
	gtk_entry_set_text(GTK_ENTRY(browser_entry), browser);
	gtk_box_pack_start(GTK_BOX(vbox), browser_entry, FALSE, FALSE, 0);
	label = gtk_label_new(_("Notify (sound) program:"));
	gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);
	notify_entry = gtk_entry_new();
	gtk_entry_set_max_length(GTK_ENTRY(notify_entry), 511);
	gtk_entry_set_text(GTK_ENTRY(notify_entry), notify);
	gtk_box_pack_start(GTK_BOX(vbox), notify_entry, FALSE, FALSE, 0);

	/* info tab */
	vbox = gkrellm_create_tab(tabs, "Info");
	scrolled = gtk_scrolled_window_new(NULL, NULL);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled),
				       GTK_POLICY_AUTOMATIC,
				       GTK_POLICY_AUTOMATIC);
	gtk_box_pack_start(GTK_BOX(vbox), scrolled, TRUE, TRUE, 0);
	text = gtk_text_new(NULL, NULL);
	gkrellm_add_info_text(text, info_text, sizeof(info_text) /
			      sizeof(gchar *));
	gtk_container_add(GTK_CONTAINER(scrolled), text);
}

/* apply_plugin_config: apply configuration settings */
static void apply_plugin_config()
{
	char *charptr;

	/* if username changed, set it and redo login */
	charptr =
	    gtk_editable_get_chars(GTK_EDITABLE(username_entry), 0, -1);
	if (strcmp(username, charptr)) {
		strcpy(username, charptr);
		relogin = TRUE;
	}
	g_free(charptr);

	/* if password changed, set and encrypt it and redo login */
	charptr =
	    gtk_editable_get_chars(GTK_EDITABLE(password_entry), 0, -1);
	if (strcmp(password, charptr)) {
		strcpy(password, charptr);
		encrypt_password();
		relogin = TRUE;
	}
	g_free(charptr);

	/* set browser command */
	charptr =
	    gtk_editable_get_chars(GTK_EDITABLE(browser_entry), 0, -1);
	strcpy(browser, charptr);
	g_free(charptr);

	/* set notify command */
	charptr =
	    gtk_editable_get_chars(GTK_EDITABLE(notify_entry), 0, -1);
	strcpy(notify, charptr);
	g_free(charptr);
}

/* save_plugin_config: write configuration to file given to us by gkrellm */
static void save_plugin_config(FILE *f)
{
	fprintf(f,
		"gklj username %s\n"
		"gklj hpassword %s\n"
		"gklj browser %s\n"
		"gklj notify %s\n", username, password, browser, notify);
}

/* load_plugin_config: parse a line from gkrellm's config */
static void load_plugin_config(gchar *arg)
{
	gchar field[10], data[512];

	if (sscanf(arg, "%s %[^\n]", field, data) == 2) {
		if (!strcmp(field, "username"))
			sscanf(data, "%63[^\n]", username);
		else if (!strcmp(field, "hpassword"))
			sscanf(data, "%32[^\n]", password);
		else if (!strcmp(field, "browser"))
			sscanf(data, "%511[^\n]", browser);
		else if (!strcmp(field, "notify"))
			sscanf(data, "%511[^\n]", notify);
	}
}

/* panel_expose_event: draw the panel */
static gint panel_expose_event(GtkWidget *widget, GdkEventExpose *ev)
{
	gdk_draw_pixmap(widget->window,
			widget->style->fg_gc[GTK_WIDGET_STATE(widget)],
			panel->pixmap, ev->area.x, ev->area.y,
			ev->area.x, ev->area.y, ev->area.width,
			ev->area.height);

	return FALSE;
}

/* panel_button_press_event: respond to mouse clicks */
static gint panel_button_press_event(GtkWidget* idget,
				     GdkEventButton *ev)
{
	led_data = D_MISC_LED0;
	strcpy(text_data, "");
	redraw();

	/* launch browser if it was a left click */
	if (ev->button == 1 && strlen(browser))
		system(browser);

	g_string_assign(lastupdate, "");
	paused = FALSE;
	if (state == STATE_CHECKFRIENDS_WAIT)
		interval = 0;

	return FALSE;
}

/* redraw: render decals based on global variables led_data and text_data */
static void redraw(void)
{
	gkrellm_draw_decal_pixmap(panel, led, led_data);
	gkrellm_draw_decal_text(panel, text, text_data, -1);
	gkrellm_draw_layers(panel);
}

/* prepare_socket: create and nonblock a socket for communication with LJ */
gboolean prepare_socket(int *s, struct hostent *he)
{
	if ((*s = socket(he->h_addrtype, SOCK_STREAM, 0)) < 0) {
		/* can't create socket */
#ifdef GKLJ_DEBUG
		strcpy(text_data, "socket");
		redraw();
		perror("socket");
#endif
		return TRUE;
	}

	if (fcntl(*s, F_SETFL, fcntl(*s, F_GETFL, 0) | O_NONBLOCK) < 0) {
		/* can't nonblock socket */
#ifdef GKLJ_DEBUG
		strcpy(text_data, "fcntl");
		redraw();
		perror("fcntl");
#endif
		close(*s);
		return TRUE;
	}

	return FALSE;
}

/* request: construct an HTTP POST request for livejournal */
static char *request(gint argc, ...)
{
	va_list arg;
	GString *tempbuf;

	va_start(arg, argc);
	tempbuf = g_string_new("");
	g_string_assign(result, "");

	/* construct the body of the request */
	while (argc--) {
		g_string_append(tempbuf, htmlesc(va_arg(arg, char *)));
		if (argc % 2)
			g_string_append_c(tempbuf, '=');
		else
			g_string_append_c(tempbuf, '&');
	}
	g_string_truncate(tempbuf, tempbuf->len - 1);

	/* construct full request */
	g_string_sprintf(result, "%s%d%s%s%s%s%s",
			 "POST /cgi-bin/log.cgi HTTP/1.0\r\n"
			 "Host: " LIVEJOURNAL_HOST "\r\n"
			 "Content-type: application/x-www-form-urlencoded\r\n"
			 "Content-length: ", tempbuf->len, "\r\n",
			 (fastserver ? "Cookie: ljfastserver=1\r\n" : ""),
			 "\r\n",
			 tempbuf->str, "\r\n"
			 "\r\n");

	g_string_free(tempbuf, TRUE);
	va_end(arg);

	return result->str;
}

/* htmlesc: escape a string to be transferred via HTTP */
static char *htmlesc(char *str)
{
	gchar byte;
	static gchar hex[4] = "%";

	g_string_assign(result, "");

	while ((byte = *(str++))) {
		if ((byte >= '0' && byte <= '9') || (byte >= 'A' && byte <=
						     'Z') || (byte >= 'a'
							      && byte <=
							      'z'))
			g_string_append_c(result, byte);
		else if (byte == ' ')
			g_string_append_c(result, '+');
		else {
			sprintf(hex + 1, "%.2X", byte);
			g_string_append(result, hex);
		}
	}

	return result->str;
}

/* parse_login: read livejournal's response to login request */
static gboolean parse_login(char *str)
{
	GString *buf;
	enum Parse parse = PARSE_NONE;
	gboolean retval = TRUE;
	GtkWidget *window, *hbox, *label, *button;

	buf = g_string_new("");
	fastserver = 0;

	while (*str) {
		if (*str == '\n') {
			if (parse == PARSE_NONE) {
				if (!buf->len)
					parse = PARSE_TYPE;
			} else if (parse == PARSE_IGNORE) {
				parse = PARSE_TYPE;
			} else if (parse == PARSE_TYPE) {
				if (!strcmp(buf->str, "success"))
					parse = PARSE_SUCCESS;
				else if (!strcmp(buf->str, "message"))
					parse = PARSE_MESSAGE;
				else if (!strcmp(buf->str, "fastserver"))
					parse = PARSE_FASTSERVER;
				else
					parse = PARSE_IGNORE;
			} else if (parse == PARSE_SUCCESS) {
				if (!strcmp(buf->str, "OK"))
					retval = FALSE;
				parse = PARSE_TYPE;
			} else if (parse == PARSE_MESSAGE) {
				window = gtk_dialog_new();
				gtk_window_set_title(GTK_WINDOW(window),
						     "LiveJournal");
				hbox = gtk_hbox_new(FALSE, 0);
				gtk_container_border_width(GTK_CONTAINER
							   (hbox), 10);
				gtk_container_add(GTK_CONTAINER
						  (GTK_DIALOG(window)->
						   vbox), hbox);
				label = gtk_label_new(buf->str);
				gtk_container_add(GTK_CONTAINER(hbox),
						  label);
				button =
				    gtk_button_new_with_label("Close");
				gtk_signal_connect_object(GTK_OBJECT
							  (button),
							  "clicked",
							  GTK_SIGNAL_FUNC
							  (gtk_widget_destroy),
							  GTK_OBJECT
							  (window));
				gtk_container_add(GTK_CONTAINER
						  (GTK_DIALOG(window)->
						   action_area), button);
				gtk_widget_show_all(window);
				parse = PARSE_TYPE;
			} else if (parse == PARSE_FASTSERVER) {
				if (!strcmp(buf->str, "1"))
					fastserver = 1;
				parse = PARSE_TYPE;
			}

			g_string_assign(buf, "");
		} else if (*str != '\r')
			g_string_append_c(buf, *str);

		++str;
	}

	g_string_free(buf, TRUE);
	return retval;
}

/* parse_checkfriends: read livejournal's response to checkfriends request */
static gboolean parse_checkfriends(char *str)
{
	GString *buf;
	enum Parse parse = PARSE_NONE;
	gboolean retval = TRUE;

	buf = g_string_new("");

	while (*str) {
		if (*str == '\n') {
			if (parse == PARSE_NONE) {
				if (!buf->len)
					parse = PARSE_TYPE;
			} else if (parse == PARSE_IGNORE) {
				parse = PARSE_TYPE;
			} else if (parse == PARSE_TYPE) {
				if (!strcmp(buf->str, "success"))
					parse = PARSE_SUCCESS;
				else if (!strcmp(buf->str, "lastupdate"))
					parse = PARSE_LASTUPDATE;
				else if (!strcmp(buf->str, "new"))
					parse = PARSE_NEW;
				else if (!strcmp(buf->str, "interval"))
					parse = PARSE_INTERVAL;
				else
					parse = PARSE_IGNORE;
			} else if (parse == PARSE_SUCCESS) {
				if (!strcmp(buf->str, "OK"))
					retval = FALSE;
				parse = PARSE_TYPE;
			} else if (parse == PARSE_LASTUPDATE) {
				g_string_assign(lastupdate, buf->str);
				parse = PARSE_TYPE;
			} else if (parse == PARSE_NEW) {
				if (strtol(buf->str, NULL, 10)) {
					/* new message */
					led_data = D_MISC_LED1;
					strcpy(text_data, "new");
					redraw();
					if (strlen(notify))
						system(notify);
					paused = TRUE;
				} else {
					strcpy(text_data, "");
					redraw();
				}
				parse = PARSE_TYPE;
			} else if (parse == PARSE_INTERVAL) {
				interval = strtol(buf->str, NULL, 10);
				if (interval < MIN_INTERVAL)
					interval = MIN_INTERVAL;
				parse = PARSE_TYPE;
			}

			g_string_assign(buf, "");
		} else if (*str != '\r')
			g_string_append_c(buf, *str);

		++str;
	}

	g_string_free(buf, TRUE);
	return retval;
}

/* encrypt_password: convert global variable password to an md5 hash */
static void encrypt_password()
{
	unsigned char digest[16];
	struct md5_ctx ctx;
	int i;

	md5_init_ctx(&ctx);
	md5_process_bytes(password, strlen(password), &ctx);
	md5_finish_ctx(&ctx, &digest);
	for (i = 0; i < 16; i++)
		sprintf(&password[i * 2], "%02x", digest[i]);
}
